#include <memory.h>

.intel_syntax noprefix


.section .bss
.align PAGE_SIZE
.skip PAGE_SIZE
BootStack:

.section .text
.code32

.global _start
_start:

    cli

    mov edi, eax
    mov esi, ebx

    //; Temporary stack for booting
    mov esp, offset V2P(BootStack)

    //; Enable Page Address Extension
    mov eax, cr4
    or eax, 1<<5
    mov cr4, eax

    //; Load Page Table (defined in boot/boot_PT.S)
    mov eax, offset V2P(BootP4)
    mov cr3, eax

    //; Enable Long Mode
    mov ecx, 0xC0000080
    rdmsr
    or eax, 1<<8
    wrmsr

    // Enable paging
    mov eax, cr0
    or eax, 1<<31
    mov cr0, eax

    //; Load GDT (defined in cpu/gdt.c)
    lgdt [V2P(GDTp)]

    //; Jump into long mode
    jmp 0x8:V2P(long_mode_start)

.code64
long_mode_start:

    //; Clear out selectors
    mov eax, 0x0
    mov ss, eax
    mov ds, eax
    mov es, eax

    movabs rax, offset upper_memory
    jmp rax

upper_memory:
    movabs rax, offset BootStack
    mov rsp, rax

    mov rax, 0
    movabs [BootP4], rax

    mov rax, cr3
    mov cr3, rax

    movabs rax, offset GDTp
    lgdt [rax]
    mov rax, 0
    mov ss, eax
    mov ds, eax
    mov es, eax

    movabs rax, offset .reload_cs
    pushq 0x8
    push rax
    retfq

.reload_cs:

.extern kmain //; boot/kmain.c
    movabs rax, offset kmain
    call rax
    
    hlt
    jmp $